@@ -60,16 +60,13 @@ class Forecast(Workflow, ModelSQL, ModelView):
         cls._sql_constraints += [
             ('check_from_to_date',
              'CHECK(to_date >= from_date)',
-             '"To Date" must be greater than "From Date"!'),
-            ]
-        cls._constraints += [
-            ('check_date_overlap', 'date_overlap'),
+             '"To Date" must be greater than "From Date"'),
             ]
         cls._error_messages.update({
-                'date_overlap': 'You can not create forecasts for the same ' \
-                    'locations with overlapping dates',
-                'delete_cancel': 'Forecast "%s" must be cancelled before '\
-                    'deletion!',
+                'date_overlap': ('Forecast "%(first)s" overlaps with dates '
+                    'of forecast "%(second)s" in the same location.'),
+                'delete_cancel': ('Forecast "%s" must be cancelled before '
+                    'deletion.'),
                 })
         cls._order.insert(0, ('from_date', 'DESC'))
         cls._order.insert(1, ('warehouse', 'ASC'))
@@ -152,31 +149,38 @@ class Forecast(Workflow, ModelSQL, ModelView):
     def default_company():
         return Transaction().context.get('company')
 
+    @classmethod
+    def validate(cls, forecasts):
+        super(Forecast, cls).validate(forecasts)
+        for forecast in forecasts:
+            forecast.check_date_overlap()
+
     def check_date_overlap(self):
         cursor = Transaction().cursor
         if self.state != 'done':
             return True
-        cursor.execute('SELECT id ' \
-                'FROM stock_forecast ' \
-                'WHERE ((from_date <= %s AND to_date >= %s) ' \
-                        'OR (from_date <= %s AND to_date >= %s) ' \
-                        'OR (from_date >= %s AND to_date <= %s)) ' \
-                    'AND warehouse = %s ' \
-                    'AND destination = %s ' \
-                    'AND state = \'done\' ' \
-                    'AND company = %s '
-                    'AND id != %s',
-                (self.from_date, self.from_date,
-                 self.to_date, self.to_date,
-                 self.from_date, self.to_date,
-                 self.warehouse.id, self.destination.id,
-                 self.company.id, self.id))
-        rowcount = cursor.rowcount
-        if rowcount == -1 or rowcount is None:
-            rowcount = len(cursor.fetchall())
-        if rowcount:
-            return False
-        return True
+        cursor.execute('SELECT id '
+            'FROM stock_forecast '
+            'WHERE ((from_date <= %s AND to_date >= %s) '
+                    'OR (from_date <= %s AND to_date >= %s) '
+                    'OR (from_date >= %s AND to_date <= %s)) '
+                'AND warehouse = %s '
+                'AND destination = %s '
+                'AND state = \'done\' '
+                'AND company = %s '
+                'AND id != %s',
+            (self.from_date, self.from_date,
+             self.to_date, self.to_date,
+             self.from_date, self.to_date,
+             self.warehouse.id, self.destination.id,
+             self.company.id, self.id))
+        forecast_id = cursor.fetchone()
+        if forecast_id:
+            second = self.__class__(forecast_id[0])
+            self.raise_user_error('date_overlap', {
+                    'first': self.rec_name,
+                    'second': second.rec_name,
+                    })
 
     @classmethod
     def delete(self, forecasts):
@@ -285,12 +289,12 @@ class ForecastLine(ModelSQL, ModelView):
         super(ForecastLine, cls).__setup__()
         cls._sql_constraints += [
             ('check_line_qty_pos',
-             'CHECK(quantity >= 0.0)', 'Line quantity must be positive!'),
+             'CHECK(quantity >= 0.0)', 'Line quantity must be positive'),
             ('check_line_minimal_qty',
              'CHECK(quantity >= minimal_quantity)',
-             'Line quantity must be greater than the minimal quantity!'),
+             'Line quantity must be greater than the minimal quantity'),
             ('forecast_product_uniq', 'UNIQUE(forecast, product)',
-             'Product must be unique by forcast!'),
+             'Product must be unique by forecast'),
             ]
 
     @staticmethod
@@ -407,13 +411,13 @@ class ForecastLine(ModelSQL, ModelView):
             unit_price = Uom.compute_price(self.product.default_uom,
                 unit_price, self.uom)
 
-        moves = []
+        to_create = []
         for day, qty in distribution.iteritems():
             if qty == 0.0:
                 continue
-            move = Move.create({
-                    'from_location': \
-                        self.forecast.warehouse.storage_location.id,
+            to_create.append({
+                    'from_location': (
+                        self.forecast.warehouse.storage_location.id),
                     'to_location': self.forecast.destination.id,
                     'product': self.product.id,
                     'uom': self.uom.id,
@@ -424,7 +428,9 @@ class ForecastLine(ModelSQL, ModelView):
                     'currency': self.forecast.company.currency.id,
                     'unit_price': unit_price,
                     })
-            moves.append(move)
+        moves = []
+        if to_create:
+            moves = Move.create(to_create)
         self.write([self], {'moves': [('set', [m.id for m in moves])]})
 
     @classmethod
@@ -490,13 +496,13 @@ class ForecastComplete(Wizard):
     __name__ = 'stock.forecast.complete'
     start_state = 'ask'
     ask = StateView('stock.forecast.complete.ask',
-        'stock_forecast.forecast_comlete_ask_view_form', [
+        'stock_forecast.forecast_complete_ask_view_form', [
             Button('Cancel', 'end', 'tryton-cancel'),
             Button('Choose Products', 'choose', 'tryton-go-next'),
             Button('Complete', 'complete', 'tryton-ok', default=True),
             ])
     choose = StateView('stock.forecast.complete.choose',
-        'stock_forecast.forecast_comlete_choose_view_form', [
+        'stock_forecast.forecast_complete_choose_view_form', [
             Button('Cancel', 'end', 'tryton-cancel'),
             Button('Choose Dates', 'ask', 'tryton-go-previous'),
             Button('Complete', 'complete', 'tryton-ok', default=True),
@@ -507,8 +513,8 @@ class ForecastComplete(Wizard):
     def __setup__(cls):
         super(ForecastComplete, cls).__setup__()
         cls._error_messages.update({
-            'from_to_date': '"From Date" should be smaller than "To Date"!',
-            })
+                'from_to_date': '"From Date" should be smaller than "To Date".',
+                })
 
     def default_ask(self, fields):
         """
@@ -542,7 +548,7 @@ class ForecastComplete(Wizard):
         Collect products for which there is an outgoing stream between
         the given location and the destination.
         """
-        if self.choose.products:
+        if getattr(self.choose, 'products', None):
             return {'products': [x.id for x in self.choose.products]}
         pbl = self._get_product_quantity()
         products = []
@@ -569,11 +575,12 @@ class ForecastComplete(Wizard):
         for product in Product.browse(product_ids):
             prod2uom[product.id] = product.default_uom.id
 
-        if self.choose.products:
+        if getattr(self.choose, 'products', None):
             products = [x.id for x in self.choose.products]
         else:
             products = None
 
+        to_create = []
         for key, qty in pbl.iteritems():
             _, product = key
             if products and product not in products:
@@ -589,11 +596,13 @@ class ForecastComplete(Wizard):
                         'minimal_quantity': min(1, -qty),
                         })
             else:
-                ForecastLine.create({
+                to_create.append({
                         'product': product,
                         'quantity': -qty,
                         'uom': prod2uom[product],
                         'forecast': Transaction().context['active_id'],
                         'minimal_quantity': min(1, -qty),
                         })
+        if to_create:
+            ForecastLine.create(to_create)
         return 'end'
